@@ -55,8 +55,6 @@ void gc_reset(void);
 size_t zend_gc_globals_size(void);
 #endif
 
-END_EXTERN_C()
-
 #define GC_REMOVE_FROM_BUFFER(p) do { \
 		zend_refcounted *_p = (zend_refcounted*)(p); \
 		if (GC_TYPE_INFO(_p) & GC_INFO_MASK) { \
@@ -66,12 +64,11 @@ END_EXTERN_C()
 
 #define GC_MAY_LEAK(ref) \
 	((GC_TYPE_INFO(ref) & \
-		(GC_INFO_MASK | (GC_COLLECTABLE << GC_FLAGS_SHIFT))) == \
-	(GC_COLLECTABLE << GC_FLAGS_SHIFT))
+		(GC_INFO_MASK | (GC_NOT_COLLECTABLE << GC_FLAGS_SHIFT))) == 0)
 
 static zend_always_inline void gc_check_possible_root(zend_refcounted *ref)
 {
-	if (EXPECTED(GC_TYPE_INFO(ref) == IS_REFERENCE)) {
+	if (EXPECTED(GC_TYPE_INFO(ref) == GC_REFERENCE)) {
 		zval *zv = &((zend_reference*)ref)->val;
 
 		if (!Z_COLLECTABLE_P(zv)) {
@@ -84,4 +81,45 @@ static zend_always_inline void gc_check_possible_root(zend_refcounted *ref)
 	}
 }
 
+/* These APIs can be used to simplify object get_gc implementations
+ * over heterogeneous structures. See zend_generator_get_gc() for
+ * a usage example. */
+
+typedef struct {
+	zval *cur;
+	zval *end;
+	zval *start;
+} zend_get_gc_buffer;
+
+ZEND_API zend_get_gc_buffer *zend_get_gc_buffer_create(void);
+ZEND_API void zend_get_gc_buffer_grow(zend_get_gc_buffer *gc_buffer);
+
+static zend_always_inline void zend_get_gc_buffer_add_zval(
+		zend_get_gc_buffer *gc_buffer, zval *zv) {
+	if (Z_REFCOUNTED_P(zv)) {
+		if (UNEXPECTED(gc_buffer->cur == gc_buffer->end)) {
+			zend_get_gc_buffer_grow(gc_buffer);
+		}
+		ZVAL_COPY_VALUE(gc_buffer->cur, zv);
+		gc_buffer->cur++;
+	}
+}
+
+static zend_always_inline void zend_get_gc_buffer_add_obj(
+		zend_get_gc_buffer *gc_buffer, zend_object *obj) {
+	if (UNEXPECTED(gc_buffer->cur == gc_buffer->end)) {
+		zend_get_gc_buffer_grow(gc_buffer);
+	}
+	ZVAL_OBJ(gc_buffer->cur, obj);
+	gc_buffer->cur++;
+}
+
+static zend_always_inline void zend_get_gc_buffer_use(
+		zend_get_gc_buffer *gc_buffer, zval **table, int *n) {
+	*table = gc_buffer->start;
+	*n = gc_buffer->cur - gc_buffer->start;
+}
+
+END_EXTERN_C()
+
 #endif /* ZEND_GC_H */
