@@ -1,6 +1,4 @@
 /*
-   +----------------------------------------------------------------------+
-   | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
    | Copyright (c) The PHP Group                                          |
    +----------------------------------------------------------------------+
@@ -19,23 +17,24 @@
 #ifdef PHP_WIN32
 typedef HANDLE php_file_descriptor_t;
 typedef DWORD php_process_id_t;
+# define PHP_INVALID_FD INVALID_HANDLE_VALUE
 #else
 typedef int php_file_descriptor_t;
 typedef pid_t php_process_id_t;
+# define PHP_INVALID_FD (-1)
 #endif
 
-/* Environment block under win32 is a NUL terminated sequence of NUL terminated
- * name=value strings.
- * Under unix, it is an argv style array.
- * */
+/* Environment block under Win32 is a NUL terminated sequence of NUL terminated
+ *   name=value strings.
+ * Under Unix, it is an argv style array. */
 typedef struct _php_process_env {
 	char *envp;
 #ifndef PHP_WIN32
 	char **envarray;
 #endif
-} php_process_env_t;
+} php_process_env;
 
-struct php_process_handle {
+typedef struct _php_process_handle {
 	php_process_id_t	child;
 #ifdef PHP_WIN32
 	HANDLE childHandle;
@@ -43,6 +42,5 @@ struct php_process_handle {
 	int npipes;
 	zend_resource **pipes;
 	char *command;
-	int is_persistent;
-	php_process_env_t env;
-};
+	php_process_env env;
+} php_process_handle;
